#include "StdAfx.h"
#include "AG_SPIMCodeGenerator.h"
#include <sstream>
using namespace std;

namespace TinyPascal { namespace Emitter {
	CAG_SPIMCodeGenerator::CAG_SPIMCodeGenerator(void) : m_LabelID(0)
{
}

CAG_SPIMCodeGenerator::~CAG_SPIMCodeGenerator(void)
{
}

string CAG_SPIMCodeGenerator::GetProgram()
{
	string strRes;

	list<string>::const_iterator iter = m_Program.begin();
	for(iter; iter!=m_Program.end(); ++iter)
		strRes += *iter + "\n";

	return strRes;
}

string CAG_SPIMCodeGenerator::GetLabel()
{	
	stringstream strStream;
	strStream << "label_" << m_LabelID;
	
	++m_LabelID;
	return strStream.str();
}
bool CAG_SPIMCodeGenerator::PushIdentifier(CAG_TPSymbolTable::TreeNodeType& Identifier)
{
	m_Program.push_back("# ----- BEGIN: Push Identifier");

	// Get identifier
	string strIdentifier(Identifier.value.begin(),
						 Identifier.value.end());
	strIdentifier = "_" + strIdentifier;

	// Push identifier onto the stack
	if(Identifier.value.value() == CAG_TPSymbolTable::integer)
	{
		m_Program.push_back("    lw $t0," + strIdentifier);
		m_Program.push_back("    add $sp, $sp, -4");
		m_Program.push_back("    sw $t0,($sp)");
	}
	else
	{
		m_Program.push_back("    l.s $f0," + strIdentifier);
		m_Program.push_back("    add $sp, $sp, -4");
		m_Program.push_back("    s.s $f0,($sp)");
	}

	m_Program.push_back("# ----- END  : Push Identifier");
	return true;
}

bool CAG_SPIMCodeGenerator::PushIntegerFactor(CAG_TPSymbolTable::TreeNodeType& IntegerFactor)
{
	m_Program.push_back("# ----- BEGIN: Push Integer Factor");

	// Get Factor
	string strIntegerFactor(IntegerFactor.value.begin(),
						    IntegerFactor.value.end());

	m_Program.push_back("    li $t0," + strIntegerFactor);
	m_Program.push_back("    add $sp, $sp, -4");
	m_Program.push_back("    sw $t0,($sp)");

	m_Program.push_back("# ----- END  : Push Integer Factor");
	return true;
}

bool CAG_SPIMCodeGenerator::PushRealFactor(CAG_TPSymbolTable::TreeNodeType& RealFactor)
{
	m_Program.push_back("# ----- BEGIN: Push Real Factor");

	// Get Factor
	string strRealFactor(RealFactor.value.begin(),
				 	     RealFactor.value.end());

	m_Program.push_back("    li.s $f0," + strRealFactor);
	m_Program.push_back("    add $sp, $sp, -4");
	m_Program.push_back("    s.s $f0,($sp)");

	m_Program.push_back("# ----- END  : Push Real Factor");
	return true;
}

bool CAG_SPIMCodeGenerator::EmitAssignment(CAG_TPSymbolTable::TreeChildrenType& OpAssign)
{
	m_Program.push_back("# ----- BEGIN: Emit Assignment");

	/*
		Assignment is emitted in two steps:
		1. First emit right-hand side of the assignment
		2. Then pop the result from stack and store it into left-hand side
	*/
	switch(OpAssign[1].value.id().to_long())
	{
	case CAG_TinyPascalGrammar::ExpressionID:
		EmitExpression(OpAssign[1]);
		break;
	case CAG_TinyPascalGrammar::TermID:
		EmitTerm(OpAssign[1]);
		break;
	case CAG_TinyPascalGrammar::RealFactorID:
		PushRealFactor(OpAssign[1]);
		break;
	case CAG_TinyPascalGrammar::IntegerFactorID:
		PushIntegerFactor(OpAssign[1]);
		break;
	case CAG_TinyPascalGrammar::IdentifierID:
		PushIdentifier(OpAssign[1]);
		break;
	}

	// Get left hand side variable
	string strIdentifier(OpAssign[0].value.begin(),
						 OpAssign[0].value.end());
	strIdentifier = "_" + strIdentifier;

	// Pop the result from stack and store it into identifier
	if(OpAssign[1].value.value() == CAG_TPSymbolTable::integer)
	{
		if(OpAssign[0].value.value() == CAG_TPSymbolTable::integer)
		{
			// Integer := Integer
			m_Program.push_back("    lw $t0,($sp)");
			m_Program.push_back("    add $sp, $sp, 4");
			m_Program.push_back("    sw $t0," + strIdentifier);
		}
		else
		{
			// Real := Integer -> we have to convert integer to real
			m_Program.push_back("    l.s $f0,($sp)");
			m_Program.push_back("    add $sp, $sp, 4");
			m_Program.push_back("    cvt.s.w $f0, $f0");
			m_Program.push_back("    s.s $f0," + strIdentifier);
		}
	}
	else
	{
		if(OpAssign[0].value.value() == CAG_TPSymbolTable::integer)
		{
			// Integer := Real -> we have to convert real to integer
			m_Program.push_back("    l.s $f0,($sp)");
			m_Program.push_back("    add $sp, $sp, 4");
			m_Program.push_back("    cvt.w.s $f0, $f0");
			m_Program.push_back("    mfc1 $t0, $f0");
			m_Program.push_back("    sw $t0," + strIdentifier);
		}
		else
		{
			// Real := Real
			m_Program.push_back("    l.s $f0,($sp)");
			m_Program.push_back("    add $sp, $sp, 4");
			m_Program.push_back("    s.s $f0," + strIdentifier);
		}
	}

	

	m_Program.push_back("# ----- END  : Emit Assignment");
	return true;
}

bool CAG_SPIMCodeGenerator::EmitTerm(CAG_TPSymbolTable::TreeNodeType& Term)
{
	m_Program.push_back("# ----- BEGIN: Emit Term");

	/*
		Every term has at most two children. Once we evaluate both children,
		we are ready to evaluate the term.
	*/
	for(size_t i=0; i<Term.children.size(); ++i)
	{
		switch(Term.children[i].value.id().to_long())
		{
			case CAG_TinyPascalGrammar::TermID:
				EmitTerm(Term.children[i]);
				break;
			case CAG_TinyPascalGrammar::ExpressionID:
				EmitExpression(Term.children[i]);
				break;
			case CAG_TinyPascalGrammar::RealFactorID:
				PushRealFactor(Term.children[i]);
				break;
			case CAG_TinyPascalGrammar::IntegerFactorID:
				PushIntegerFactor(Term.children[i]);
				break;
			case CAG_TinyPascalGrammar::IdentifierID:
				PushIdentifier(Term.children[i]);
				break;
		}
	}

	/* Check what it is */
	string strOp(Term.value.begin(),
		         Term.value.end());
	if(strOp == "*")
		EmitMultiplication(Term);
	else EmitDivision(Term);

	m_Program.push_back("# ----- END  : Emit Term");
	return true;
}

bool CAG_SPIMCodeGenerator::EmitExpression(CAG_TPSymbolTable::TreeNodeType& Expression)
{
	m_Program.push_back("# ----- BEGIN: Emit Expression");

	/*
		Every term has at most two children. Once we evaluate both children,
		we are ready to evaluate the term.
	*/
	for(size_t i=0; i<Expression.children.size(); ++i)
	{
		switch(Expression.children[i].value.id().to_long())
		{
			case CAG_TinyPascalGrammar::TermID:
				EmitTerm(Expression.children[i]);
				break;
			case CAG_TinyPascalGrammar::ExpressionID:
				EmitExpression(Expression.children[i]);
				break;
			case CAG_TinyPascalGrammar::RealFactorID:
				PushRealFactor(Expression.children[i]);
				break;
			case CAG_TinyPascalGrammar::IntegerFactorID:
				PushIntegerFactor(Expression.children[i]);
				break;
			case CAG_TinyPascalGrammar::IdentifierID:
				PushIdentifier(Expression.children[i]);
				break;
		}
	}

	/* Check what it is */
	string strOp(Expression.value.begin(),
		         Expression.value.end());
	if(strOp == "+")
		EmitAddition(Expression);
	else EmitSubtraction(Expression);

	m_Program.push_back("# ----- END  : Emit Expression");
	return true;
}

bool CAG_SPIMCodeGenerator::EmitAddition(CAG_TPSymbolTable::TreeNodeType& Addition)
{
	m_Program.push_back("# ----- BEGIN: Emit Addition");

	/* 
		Addition is done by popping both operands from stack 
		and adding them. Result is pushed onto stack. This operation
		must be performed with type conversion in mind.
	*/
	if(Addition.value.value() == CAG_TPSymbolTable::integer)
	{
		/*
			This means that result is integer. If result is integer, we know that
			both operands must be integers.
		*/
		m_Program.push_back("    lw $t0,($sp)");
		m_Program.push_back("    add $sp, $sp, 4");
		m_Program.push_back("    lw $t1,($sp)");
		m_Program.push_back("    add $t0, $t1, $t0");
		m_Program.push_back("    sw $t0, ($sp)");
	}
	else
	{
		/* Result is real - operands can be mixed in any combination */
		if(Addition.children[0].value.value() == CAG_TPSymbolTable::integer)
		{
			/* 
				First operand is integer, then second operand must be real 
			    (but they are pushed onto stack in reverse order)
			*/
			m_Program.push_back("    l.s $f0,($sp)");
			m_Program.push_back("    add $sp, $sp, 4");
			m_Program.push_back("    l.s $f1, ($sp)");
			m_Program.push_back("    add $sp, $sp, 4");
			m_Program.push_back("    cvt.s.w $f1, $f1");
			m_Program.push_back("    add.s $f0, $f1, $f0");
			m_Program.push_back("    s.s $f0, ($sp)");
		} 
		else if(Addition.children[1].value.value() == CAG_TPSymbolTable::integer)
		{
			/* 
				First operand is not integer, but second operand is.
			*/
			m_Program.push_back("    l.s $f0,($sp)");
			m_Program.push_back("    add $sp, $sp, 4");
			m_Program.push_back("    cvt.s.w $f0, $f0");
			m_Program.push_back("    l.s $f1, ($sp)");
			m_Program.push_back("    add $sp, $sp, 4");
			m_Program.push_back("    add.s $f0, $f1, $f0");
			m_Program.push_back("    s.s $f0, ($sp)");
		}
		else
		{
			/* Both operands are real */
			m_Program.push_back("    l.s $f0, ($sp)");
			m_Program.push_back("    add $sp, $sp, 4");
			m_Program.push_back("    l.s $f1, ($sp)");
			m_Program.push_back("    add.s $f0, $f1, $f0");
			m_Program.push_back("    s.s $f0, ($sp)");
		}
	}

	m_Program.push_back("# ----- END  : Emit Addition");
	return true;
}

bool CAG_SPIMCodeGenerator::EmitSubtraction(CAG_TPSymbolTable::TreeNodeType& Subtraction)
{
	m_Program.push_back("# ----- BEGIN: Emit Subtraction");

	/* 
		Subtraction is done by popping both operands from stack 
		and subtracting them. Result is pushed onto stack. This operation
		must be performed with type conversion in mind.
	*/
	if(Subtraction.value.value() == CAG_TPSymbolTable::integer)
	{
		/*
			This means that result is integer. If result is integer, we know that
			both operands must be integers.
		*/
		m_Program.push_back("    lw $t0,($sp)");
		m_Program.push_back("    add $sp, $sp, 4");
		m_Program.push_back("    lw $t1,($sp)");
		m_Program.push_back("    sub $t0, $t1, $t0");
		m_Program.push_back("    sw $t0, ($sp)");
	}
	else
	{
		/* Result is real - operands can be mixed in any combination */
		if(Subtraction.children[0].value.value() == CAG_TPSymbolTable::integer)
		{
			/* 
				First operand is integer, then second operand must be real 
			    (but they are pushed onto stack in reverse order)
			*/
			m_Program.push_back("    l.s $f0,($sp)");
			m_Program.push_back("    add $sp, $sp, 4");
			m_Program.push_back("    l.s $f1, ($sp)");
			m_Program.push_back("    add $sp, $sp, 4");
			m_Program.push_back("    cvt.s.w $f1, $f1");
			m_Program.push_back("    sub.s $f0, $f1, $f0");
			m_Program.push_back("    s.s $f0, ($sp)");
		} 
		else if(Subtraction.children[1].value.value() == CAG_TPSymbolTable::integer)
		{
			/* 
				First operand is not integer, but second operand is.
			*/
			m_Program.push_back("    l.s $f0,($sp)");
			m_Program.push_back("    add $sp, $sp, 4");
			m_Program.push_back("    cvt.s.w $f0, $f0");
			m_Program.push_back("    l.s $f1, ($sp)");
			m_Program.push_back("    add $sp, $sp, 4");
			m_Program.push_back("    sub.s $f0, $f1, $f0");
			m_Program.push_back("    s.s $f0, ($sp)");
		}
		else
		{
			/* Both operands are real */
			m_Program.push_back("    l.s $f0, ($sp)");
			m_Program.push_back("    add $sp, $sp, 4");
			m_Program.push_back("    l.s $f1, ($sp)");
			m_Program.push_back("    sub.s $f0, $f1, $f0");
			m_Program.push_back("    s.s $f0, ($sp)");
		}
	}

	m_Program.push_back("# ----- END  : Emit Subtraction");
	return true;
}

bool CAG_SPIMCodeGenerator::EmitDivision(CAG_TPSymbolTable::TreeNodeType& Division)
{
	m_Program.push_back("# ----- BEGIN: Emit Division");

	/* 
		Division is done by popping both operands from stack 
		and dividing them. Result is pushed onto stack. This operation
		must be performed with type conversion in mind.
	*/
	if(Division.value.value() == CAG_TPSymbolTable::integer)
	{
		/*
			This means that result is integer. If result is integer, we know that
			both operands must be integers.
		*/
		m_Program.push_back("    lw $t0,($sp)");
		m_Program.push_back("    add $sp, $sp, 4");
		m_Program.push_back("    lw $t1,($sp)");
		m_Program.push_back("    div $t1, $t0");
		m_Program.push_back("    mflo $t0");
		m_Program.push_back("    sw $t0, ($sp)");
	}
	else
	{
		/* Result is real - operands can be mixed in any combination */
		if(Division.children[0].value.value() == CAG_TPSymbolTable::integer)
		{
			/* 
				First operand is integer, then second operand must be real 
			    (but they are pushed onto stack in reverse order)
			*/
			m_Program.push_back("    l.s $f0,($sp)");
			m_Program.push_back("    add $sp, $sp, 4");
			m_Program.push_back("    l.s $f1, ($sp)");
			m_Program.push_back("    add $sp, $sp, 4");
			m_Program.push_back("    cvt.s.w $f1, $f1");
			m_Program.push_back("    div.s $f0, $f1, $f0");
			m_Program.push_back("    s.s $f0, ($sp)");
		} 
		else if(Division.children[1].value.value() == CAG_TPSymbolTable::integer)
		{
			/* 
				First operand is not integer, but second operand is.
			*/
			m_Program.push_back("    l.s $f0,($sp)");
			m_Program.push_back("    add $sp, $sp, 4");
			m_Program.push_back("    cvt.s.w $f0, $f0");
			m_Program.push_back("    l.s $f1, ($sp)");
			m_Program.push_back("    add $sp, $sp, 4");
			m_Program.push_back("    div.s $f0, $f1, $f0");
			m_Program.push_back("    s.s $f0, ($sp)");
		}
		else
		{
			/* Both operands are real */
			m_Program.push_back("    l.s $f0, ($sp)");
			m_Program.push_back("    add $sp, $sp, 4");
			m_Program.push_back("    l.s $f1, ($sp)");
			m_Program.push_back("    div.s $f0, $f1, $f0");
			m_Program.push_back("    s.s $f0, ($sp)");
		}
	}

	m_Program.push_back("# ----- END  : Emit Division");
	return true;
}

bool CAG_SPIMCodeGenerator::EmitMultiplication(CAG_TPSymbolTable::TreeNodeType& Multiplication)
{
	m_Program.push_back("# ----- BEGIN: Emit Multiplication");

	/* Multiplication is done by popping both operands from stack 
	   and multiplying them. Result is pushed onto stack. This operation
	   must be performed with type conversion in mind
	*/
	if(Multiplication.value.value() == CAG_TPSymbolTable::integer)
	{
		/*
			This means that result is integer. If result is integer, we know that
			both operands must be integers.
		*/
		m_Program.push_back("    lw $t0,($sp)");
		m_Program.push_back("    add $sp, $sp, 4");
		m_Program.push_back("    lw $t1,($sp)");
		m_Program.push_back("    mult $t1, $t0");
		m_Program.push_back("    mflo $t0");
		m_Program.push_back("    sw $t0, ($sp)");
	}
	else
	{
		/* Result is real - operands can be mixed in any combination */
		if(Multiplication.children[0].value.value() == CAG_TPSymbolTable::integer)
		{
			/* 
				First operand is integer, then second operand must be real 
			    (but they are pushed onto stack in reverse order)
			*/
			m_Program.push_back("    l.s $f0,($sp)");
			m_Program.push_back("    add $sp, $sp, 4");
			m_Program.push_back("    l.s $f1, ($sp)");
			m_Program.push_back("    add $sp, $sp, 4");
			m_Program.push_back("    cvt.s.w $f1, $f1");
			m_Program.push_back("    mul.s $f0, $f1, $f0");
			m_Program.push_back("    s.s $f0, ($sp)");
		} 
		else if(Multiplication.children[1].value.value() == CAG_TPSymbolTable::integer)
		{
			/* 
				First operand is not integer, but second operand is.
			*/
			m_Program.push_back("    l.s $f0,($sp)");
			m_Program.push_back("    add $sp, $sp, 4");
			m_Program.push_back("    cvt.s.w $f0, $f0");
			m_Program.push_back("    l.s $f1, ($sp)");
			m_Program.push_back("    add $sp, $sp, 4");
			m_Program.push_back("    mul.s $f0, $f1, $f0");
			m_Program.push_back("    s.s $f0, ($sp)");
		}
		else
		{
			/* Both operands are real */
			m_Program.push_back("    l.s $f0, ($sp)");
			m_Program.push_back("    add $sp, $sp, 4");
			m_Program.push_back("    l.s $f1, ($sp)");
			m_Program.push_back("    mul.s $f0, $f1, $f0");
			m_Program.push_back("    s.s $f0, ($sp)");
		}
	}

	m_Program.push_back("# ----- END  : Emit Multiplication");
	return true;
}

bool CAG_SPIMCodeGenerator::EmitWritelnStatement(CAG_TPSymbolTable::TreeChildrenType& WritelnStmt)
{
	m_Program.push_back("# ----- BEGIN: Emit Writeln Statement");

	for(size_t i=0; i<WritelnStmt.size(); ++i)
	{
		switch(WritelnStmt[i].value.id().to_long())
		{
		case CAG_TinyPascalGrammar::ExpressionID:
			EmitExpression(WritelnStmt[i]);
			break;
		case CAG_TinyPascalGrammar::TermID:
			EmitTerm(WritelnStmt[i]);
			break;
		case CAG_TinyPascalGrammar::RealFactorID:
			PushRealFactor(WritelnStmt[i]);
			break;
		case CAG_TinyPascalGrammar::IntegerFactorID:
			PushIntegerFactor(WritelnStmt[i]);
			break;
		case CAG_TinyPascalGrammar::IdentifierID:
			PushIdentifier(WritelnStmt[i]);
			break;
		}

		// Pop result from the stack and print either real or integer
		if(WritelnStmt[i].value.value() == CAG_TPSymbolTable::integer)
		{
			m_Program.push_back("    lw $a0,($sp)");
			m_Program.push_back("    add $sp, $sp, 4");
			m_Program.push_back("    li $v0, 1");
			m_Program.push_back("    syscall");
		}
		else
		{
			m_Program.push_back("    l.s $f12,($sp)");
			m_Program.push_back("    add $sp, $sp, 4");
			m_Program.push_back("    li $v0, 2");
			m_Program.push_back("    syscall");
		}

		// Print newline
		m_Program.push_back("    la $a0, writeln");
		m_Program.push_back("    li $v0, 4");
		m_Program.push_back("    syscall");
	}

	m_Program.push_back("# ----- END  : Emit Writeln Statement");
	return true;
}

bool CAG_SPIMCodeGenerator::EmitVarDecl(CAG_TPSymbolTable::SymbolTableType& SymbolTable)
{
	m_Program.push_back("# ----- BEGIN: Emit Symbol Table");

	/*
		integer = .word
		real	= .float
	*/
	m_Program.push_back("    .data");

	CAG_TPSymbolTable::SymbolTableType::const_iterator iter = SymbolTable.begin();
	for(iter; iter != SymbolTable.end(); ++iter)
	{
		if(iter->second == CAG_TPSymbolTable::real)
			m_Program.push_back("_" + iter->first + ": .float 0.0");
		else if(iter->second == CAG_TPSymbolTable::integer)
			m_Program.push_back("_" + iter->first + ": .word  0");
	}

	// Writeln statement helper
	m_Program.push_back("writeln: .asciiz \"\\n\"");

	m_Program.push_back("# ----- END  : Emit Symbol Table");
	return true;
}

bool CAG_SPIMCodeGenerator::EmitMainStatementPart(CAG_TPSymbolTable::TreeChildrenType& MainStatementPart)
{
	m_Program.push_back("# ----- BEGIN: Emit Main Statement Part");

	/*
		MainStatementPart is always surrounded by begin and end so that first
		and last child could be ommited during traversal. Following loop emits
		all children of the MainStatementPart
	*/
	for(size_t i=1; i<MainStatementPart.size()-1; ++i)
	{
		switch(MainStatementPart[i].value.id().to_long())
		{
		case CAG_TinyPascalGrammar::OpAssignID:
			EmitAssignment(MainStatementPart[i].children);
			break;
		case CAG_TinyPascalGrammar::MainStatementPartID:
			EmitMainStatementPart(MainStatementPart[i].children);
			break;
		case CAG_TinyPascalGrammar::WritelnStmtID:
			EmitWritelnStatement(MainStatementPart[i].children);
			break;
		case CAG_TinyPascalGrammar::ForStatementID:
			EmitForLoop(MainStatementPart[i].children);
			break;
		}
	}

	m_Program.push_back("# ----- END  : Emit Main Statement Part");
	return true;
}

bool CAG_SPIMCodeGenerator::EmitForLoop(CAG_TPSymbolTable::TreeChildrenType& ForLoop)
{
	/*
		For loop is emitted in following way:
		
		1.		Evaluate(OpAssign) - LoopVariable contains initial value
			LOOP: 
		2.		Evaluate(Expression) - result will be pushed on stack
		3.		Compare(Expression, LoopVariable) - if true then jump to DONE
					- If keyword "downto" is used then comparison should be LoopVariable >= Expression
					- If keyword "to" is used then comparison should be LoopVariable <= Expression.
		4.		Evaluate(Statement)
		5.		Modify LoopVariable
					- If keyword "downto" is used then Decrease(LoopVariable)
					- If keyword "to" is used then Increase(LoopVariable)
		6.		Jump LOOP
			DONE:

		NOTE: Such loop interpretation is different than "standard" Pascal, where loop
		variable cannot be changed inside the loop.

	*/
	m_Program.push_back("# ----- BEGIN: Emit For Loop");

	// Get both labels at the begin
	string strLoop = GetLabel();
	string strDone = GetLabel();

	// Get loop variable
	string strLoopVar(ForLoop[1].children[0].value.begin(),
					  ForLoop[1].children[0].value.end());
	strLoopVar = "_" + strLoopVar;

	// Determine if we are increasing or decreasing
	bool bIncrease = true;
	string strKeyword(ForLoop[2].value.begin(),
					  ForLoop[2].value.end());
	transform(strKeyword.begin(), strKeyword.end(), strKeyword.begin(), tolower);
	if(strKeyword.compare("downto") == 0)
		bIncrease = false;

	// Evaluate assignment
	EmitAssignment(ForLoop[1].children);

	// Emit loop label
	m_Program.push_back(strLoop + ":");

	// Evaluate Expression Part
	switch(ForLoop[3].value.id().to_long())
	{
	case CAG_TinyPascalGrammar::ExpressionID:
		EmitExpression(ForLoop[3]);
		break;
	case CAG_TinyPascalGrammar::TermID:
		EmitTerm(ForLoop[3]);
		break;
	case CAG_TinyPascalGrammar::RealFactorID:
		PushRealFactor(ForLoop[3]);
		break;
	case CAG_TinyPascalGrammar::IntegerFactorID:
		PushIntegerFactor(ForLoop[3]);
		break;
	case CAG_TinyPascalGrammar::IdentifierID:
		PushIdentifier(ForLoop[3]);
		break;
	}

	/*
		Comparison is perfomed in following way:
		1. Pop expression result from the stack
		2. Load loop varibale value
		3. Use branch macro to determine if to branch
	*/
	m_Program.push_back("    lw $t1,($sp)");
	m_Program.push_back("    add $sp, $sp, 4");
	m_Program.push_back("    lw $t0," + strLoopVar);
	if(bIncrease)
		m_Program.push_back("    bgt $t0, $t1," + strDone); // $t0 >= $t1
	else m_Program.push_back("    blt $t0, $t1," + strDone); // $t0 <= $t1

	// Evaluate loop statement part
	if(ForLoop.size() == 6)
	{
		switch(ForLoop[5].value.id().to_long())
		{
		case CAG_TinyPascalGrammar::OpAssignID:
			EmitAssignment(ForLoop[5].children);
			break;
		case CAG_TinyPascalGrammar::MainStatementPartID:
			EmitMainStatementPart(ForLoop[5].children);
			break;
		case CAG_TinyPascalGrammar::WritelnStmtID:
			EmitWritelnStatement(ForLoop[5].children);
			break;
		case CAG_TinyPascalGrammar::ForStatementID:
			EmitForLoop(ForLoop[5].children);
			break;
		}
	}

	// Increase/Decrease loop variable
	m_Program.push_back("    lw $t0," + strLoopVar);
	if(bIncrease)
		m_Program.push_back("    addi $t0, $t0, 1");
	else m_Program.push_back("    addi $t0, $t0, -1");
	m_Program.push_back("    sw $t0," + strLoopVar);

	// Evaluate loop again
	m_Program.push_back("    b " + strLoop);

	// Emit DONE label
	m_Program.push_back(strDone + ":");

	m_Program.push_back("# ----- END  : Emit For Loop");

	return true;
}

bool CAG_SPIMCodeGenerator::Emit(CAG_TPSymbolTable::TreeChildrenType& AST,
	                             CAG_TPSymbolTable::SymbolTableType& SymbolTable)
{
	// Clear origninal program
	m_Program.clear();

	// Initialize label id counting to 0
	m_LabelID = 0;

	// Write program name as comment
	m_strProgramName = "_" + string(AST[0].children[0].value.begin(), AST[0].children[0].value.end());
	m_Program.push_back("# " + m_strProgramName);
	m_Program.push_back("");

	// Emit variable declaration part
	EmitVarDecl(SymbolTable);

	m_Program.push_back("    .text");
	m_Program.push_back("    .globl main");
	m_Program.push_back("");
	m_Program.push_back("main:");

	// Recursivly emit main statement part
	// Find main statement part subtree
	CAG_TPSymbolTable::TreeChildrenType& MainStatementPart = CAG_TPSymbolTable::TreeChildrenType();
	for(size_t i=0; i<AST[0].children.size(); ++i)
		if(AST[0].children[i].value.id().to_long() == CAG_TinyPascalGrammar::MainStatementPartID)
			MainStatementPart = AST[0].children[i].children;
	EmitMainStatementPart(MainStatementPart);

	m_Program.push_back("    li $v0,10 # syscall to exit the program");
	m_Program.push_back("    syscall");

	return true;
}

}}