#pragma once
#include "AG_CodeGenerator.h"
#include <string>
#include <list>
using namespace std;

namespace TinyPascal { namespace Emitter {
class CAG_SPIMCodeGenerator : public CAG_CodeGenerator
{
protected:
	//! Program is simply linked list of statements
	list<string> m_Program;

	//! Contains the next label ID
	unsigned int m_LabelID;

	//! Program name
	string m_strProgramName;

	//! Returns unique label
	string GetLabel();

	//! Emits variable declaration part
	bool EmitVarDecl(CAG_TPSymbolTable::SymbolTableType& SymbolTable);

	//! Emits main statement part
	bool EmitMainStatementPart(CAG_TPSymbolTable::TreeChildrenType& MainStatementPart);

	//! Emits assignment operator
	bool EmitAssignment(CAG_TPSymbolTable::TreeChildrenType& OpAssign);

	//! Emits expression 
	bool EmitExpression(CAG_TPSymbolTable::TreeNodeType& Expression);

	//! Emits term
	bool EmitTerm(CAG_TPSymbolTable::TreeNodeType& Term);

	//! Emits addition
	bool EmitAddition(CAG_TPSymbolTable::TreeNodeType& Addition);

	//! Emits addition
	bool EmitSubtraction(CAG_TPSymbolTable::TreeNodeType& Subtraction);

	//! Emits division
	bool EmitDivision(CAG_TPSymbolTable::TreeNodeType& Division);

	//! Emits multiplication
	bool EmitMultiplication(CAG_TPSymbolTable::TreeNodeType& Multiplication);

	//! Emits Identifier by pushing it onto stack
	bool PushIdentifier(CAG_TPSymbolTable::TreeNodeType& Identifier);

	//! Emits Integer Factor by pushing it onto stack
	bool PushIntegerFactor(CAG_TPSymbolTable::TreeNodeType& Factor);

	//! Emits Real Factor by pushing it onto stack
	bool PushRealFactor(CAG_TPSymbolTable::TreeNodeType& Factor);

	//! Emits Writeln Statement
	bool EmitWritelnStatement(CAG_TPSymbolTable::TreeChildrenType& WritelnStmt);

	//! Emits FOR loop
	bool EmitForLoop(CAG_TPSymbolTable::TreeChildrenType& ForLoop);

public:
	//! Default constructor
	CAG_SPIMCodeGenerator(void);

	//! Destructor
	~CAG_SPIMCodeGenerator(void);

	//! Emit MIPS assambly code
	bool Emit(CAG_TPSymbolTable::TreeChildrenType& AST,
			  CAG_TPSymbolTable::SymbolTableType& SymbolTable);

	//! Returns whole program in a string object
	string GetProgram();

	//! Returns name of the file that needs to be created
	string GetFileName() { return m_strProgramName + ".s"; };
};
}}
