#include "StdAfx.h"
#include "AG_TPCompiler.h"
#include <fstream>
#include <iostream>
#include <sstream>
#include <ios>
using namespace std;

namespace TinyPascal {
CAG_TPCompiler::CAG_TPCompiler(void) : m_strWorkingDir("c:\\")
{
}

CAG_TPCompiler::~CAG_TPCompiler(void)
{
}

bool CAG_TPCompiler::Compile(string program)
{
	// Clear logs
	CAG_TPMessageLog::m_ErrorLog.clear();
	CAG_TPMessageLog::m_WarningLog.clear();

	// Delete AST
	m_AST.clear();

	// Grammar/Skipper objects
	Parser::CAG_TinyPascalGrammar TinyPascalGrammar;
	Parser::CAG_TinyPascalSkipper TinyPascalSkipper;

	// Create position iterator objects
	position_iterator<string::const_iterator> begin(program.begin(), program.end());
	position_iterator<string::const_iterator> end;

	// Parse and create AST, or rather parse tree with some AST features
	node_val_data_factory<Semantics::CAG_TPSymbolTable::ExpressionType> ExpressionTypeFactory;
	tree_parse_info<position_iterator<string::const_iterator>, node_val_data_factory<Semantics::CAG_TPSymbolTable::ExpressionType> > info = 
		ast_parse(begin, end, TinyPascalGrammar, TinyPascalSkipper, ExpressionTypeFactory);

	// Assign AST
	m_AST = info.trees;

	if(info.full)
	{
		if(m_SymbolTable.CreateSymbolTable(m_AST))
		{
			if(m_TypeChecker.CheckType(m_AST, m_SymbolTable))
			{
				m_JavaBytecodeGenerator.Emit(m_AST, m_SymbolTable.GetSymbolTable());
				m_SPIMCodeGenerator.Emit(m_AST, m_SymbolTable.GetSymbolTable());

				return MakeFiles();
			}
		}
		else 
		{
			return false;
		}
	}
	else
	{
		// Determine position of syntax error
		file_position error_pos = info.stop.get_position();

		stringstream str_stream;
		str_stream.setf(ios_base::dec); str_stream << "    Syntax error line " 
			<< error_pos.line << ": " << GetLine(program, error_pos.line);
		CAG_TPMessageLog::m_ErrorLog.push_back(str_stream.str());
		return false;
	}

	return false;
}


bool CAG_TPCompiler::MakeFiles()
{
	bool bRes = true;
	if(m_strWorkingDir.empty())
		return false;

	// First store java class file
	string strFileName = m_strWorkingDir + m_JavaBytecodeGenerator.GetFileName();
	ofstream class_file(strFileName.c_str(), ios_base::binary | ios_base::trunc | ios_base::out);
	if(!class_file.bad())
	{
		string strProgram = m_JavaBytecodeGenerator.GetProgram();
		class_file << strProgram;
		class_file.close();
	} else bRes = false;

	// Now store asm file
	strFileName = m_strWorkingDir + m_SPIMCodeGenerator.GetFileName();
	ofstream spim_file(strFileName.c_str(), ios_base::binary | ios_base::trunc | ios_base::out);
	if(!spim_file.bad())
	{
		string strProgram = m_SPIMCodeGenerator.GetProgram();
		spim_file << strProgram;
		spim_file.close();
	} else bRes = false;

	return bRes;
}

string CAG_TPCompiler::GetLine(string& program, int line_index)
{
	string strRes;
	int index = 1;
	string::iterator iter = program.begin();
	for(iter; iter != program.end(); ++iter)
	{
		if(*iter == '\x0A')
		{
			if(line_index == index)
				return strRes;
			else { strRes = ""; ++index; }
		}
		else strRes += *iter;
	}
	return "";
}

}
