#pragma once

#include "Grammar.h"
#include "AG_TPMessageLog.h"
#include "AG_TPSymbolTable.h"
#include "AG_TPTypeChecker.h"
#include "AG_SPIMCodeGenerator.h"
#include "AG_JavaBytecodeGenerator.h"

#include <string>
using namespace std;

namespace TinyPascal
{
class CAG_TPCompiler
{
protected:
	//! Stores compiled MIPS assembler and bytecode to a file
	bool MakeFiles();

public:
	//! Symbol table object
	Semantics::CAG_TPSymbolTable m_SymbolTable;

	//! Type checker object
	Semantics::CAG_TPTypeChecker m_TypeChecker;

	//! AST object
	Semantics::CAG_TPSymbolTable::TreeChildrenType m_AST;

	//! SPIM Simulator Assambler Emitter
	Emitter::CAG_SPIMCodeGenerator m_SPIMCodeGenerator;

	//! Java Bytecode Generator
	Emitter::CAG_JavaBytecodeGenerator m_JavaBytecodeGenerator;

	//! Compiler working directory - asm and class files will be stored here
	string m_strWorkingDir;

	//! Default constructor
	CAG_TPCompiler(void);

	//! Destructor
	~CAG_TPCompiler(void);

	//! Compiles program stored in std::string object
	bool Compile(string program);

	//! Parses the line with specified index (1-based)
	string GetLine(string& program, int line_index);
};
}
