#include "StdAfx.h"
#include "AG_TPSymbolTable.h"
#include <algorithm>

namespace TinyPascal { namespace Semantics {
CAG_TPSymbolTable::CAG_TPSymbolTable(void)
{
}

CAG_TPSymbolTable::~CAG_TPSymbolTable(void)
{
}

bool CAG_TPSymbolTable::CreateSymbolTable(TreeChildrenType Root)
{
	// Return value
	bool Res = true;
	m_SymbolTable.clear();

	// Get references to var declaration part and main statement part 
	TreeChildrenType& VarDeclarationRoot = TreeChildrenType();
	TreeChildrenType& MainStatementPartRoot = TreeChildrenType();
	for(size_t i=0; i<Root[0].children.size(); ++i)
	{
		if(Root[0].children[i].value.id().to_long() == CAG_TinyPascalGrammar::VarDeclarationID)
			VarDeclarationRoot = Root[0].children[i].children;
		else if(Root[0].children[i].value.id().to_long() == CAG_TinyPascalGrammar::MainStatementPartID)
			MainStatementPartRoot = Root[0].children[i].children;
	}

	// Scan global declaration part
	if(!TraverseVarDeclaration(VarDeclarationRoot))
		Res = false;

	// Scan main statement part recursivly
	TraverseMainStatementPart(MainStatementPartRoot, Res);

	return Res;
}

bool CAG_TPSymbolTable::TraverseVarDeclaration(TreeChildrenType& Root)
{
	// resulting value
	bool Res = true;

	// Root contains vector of all chlidren from VarDeclaration - first child is "var" keyword
	for(size_t i=1; i<Root.size(); ++i)
	{
		if(Root[i].value.id().to_long() == CAG_TinyPascalGrammar::TypeID)
		{
			// Get type
			string strType(Root[i].value.begin(), Root[i].value.end());
			transform(strType.begin(), strType.end(), strType.begin(), tolower);
			ExpressionType Type;
			if(strType.compare("integer") == 0)
				Type = integer;
			else Type = real;

			for(size_t j=0; j<Root[i].children.size(); ++j)
			{
				// Get identifier string
				string strIdentifier(Root[i].children[j].value.begin(),
									 Root[i].children[j].value.end());

				// Add new symbol
				if(!AddSymbol(strIdentifier, Type))
				{
					CAG_TPMessageLog::m_ErrorLog.push_back("    ERROR: " + strIdentifier + " already defined!");
					Res = false;
				}
			}
		}
		else
		{
			CAG_TPMessageLog::m_ErrorLog.push_back("Internal Compiler Error: VarDeclaration contains unexpected rule!");
			Res = false;
		}
	}

	return Res;
}

void CAG_TPSymbolTable::TraverseMainStatementPart(TreeChildrenType& Root, bool& Res)
{
	for(size_t i=0; i<Root.size(); ++i)
	{
		// Check is it a symbol and if it is already in symbol table
		if(Root[i].value.id().to_long() == CAG_TinyPascalGrammar::IdentifierID)
		{
			// Get identifier string
			string strIdentifier(Root[i].value.begin(),
								 Root[i].value.end());

			if(m_SymbolTable.find(strIdentifier) == m_SymbolTable.end())
			{
				AddSymbol(strIdentifier, undefined);
				CAG_TPMessageLog::m_ErrorLog.push_back("    ERROR: " + strIdentifier + " not defined!");
				Res = false;
			}
		}
		else TraverseMainStatementPart(Root[i].children, Res);
	}
}

bool CAG_TPSymbolTable::AddSymbol(string Symbol, ExpressionType Type)
{
	// Check does the symbol exist
	if(m_SymbolTable.find(Symbol) == m_SymbolTable.end())
	{
		m_SymbolTable[Symbol] = Type;
		return true;
	}
	return false;
}

CAG_TPSymbolTable::ExpressionType CAG_TPSymbolTable::GetSymbolType(string Symbol)
{
	//! Check if the symbol is in the symbol table
	if(m_SymbolTable.find(Symbol) != m_SymbolTable.end())
		return m_SymbolTable[Symbol];

	return undefined;
}

}}
