#pragma once
#include "Grammar.h"
#include "AG_TPMessageLog.h"
#include <hash_map>
#include <string>
#include "ctype.h"
using namespace std;
using namespace stdext;
using namespace TinyPascal::Parser;

namespace TinyPascal { namespace Semantics {
class CAG_TPSymbolTable
{
public:
	//! Expression type used to denote type of the expression
	enum ExpressionType
	{
		undefined,
		real,
		integer
	};

protected:
	//! Functor for case insensitive string comparison
	struct CAG_CompareNoCase
	{
		//! Case insensitive character comaparison
		static bool CompareNoCaseChar(char c1, char c2)
		{ return(toupper(c1)<toupper(c2)); }

		//! Override comparison operator
		bool operator()(const string& s1, const string& s2) const
		{
			return lexicographical_compare(s1.begin(), s1.end(),
								           s2.begin(), s2.end(),
									       CompareNoCaseChar);
		}
	};

public:
	typedef hash_map<string, ExpressionType, hash_compare<string, CAG_CompareNoCase> > SymbolTableType;

protected:
	//! Symbol table hash_maps
	SymbolTableType m_SymbolTable;

public:
	//! Specify type of the AST tree children
	typedef vector<tree_node<node_val_data<position_iterator<string::const_iterator>, ExpressionType > > > TreeChildrenType;

	//! Specify AST tree node type
	typedef tree_node<node_val_data<position_iterator<string::const_iterator>, ExpressionType > > TreeNodeType;

	//! Default constructor
	CAG_TPSymbolTable(void);

	//! Destructor
	~CAG_TPSymbolTable(void);

	//! Constructs symbol table from the root of the tree
	bool CreateSymbolTable(TreeChildrenType Root);

	//! Adds a symbol to symbol table.
	/*! Adds a symbol to symbol table.
		
		\param Symbol Symbol to be added.
		\param Type Symbol type
		\return True if symbol added. False if symbol exists already in the table.
	*/
	bool AddSymbol(string Symbol, ExpressionType Type);

	//! Returns symbol type
	ExpressionType GetSymbolType(string Symbol);

	//! Return symbol table hash_map
	SymbolTableType& GetSymbolTable()
	{
		return m_SymbolTable;
	}

protected:
	//! Scans global declaration part for variable definition
	bool TraverseVarDeclaration(TreeChildrenType& Root);

	//! Scan main statement part for undefined identifiers
	void TraverseMainStatementPart(TreeChildrenType& Root, bool& Res);
};
}}