#include "StdAfx.h"
#include "Grammar.h"
#include "AG_TPTypeChecker.h"
#include "AG_TPMessageLog.h"
using namespace TinyPascal::Parser;

namespace TinyPascal { namespace Semantics {
CAG_TPTypeChecker::CAG_TPTypeChecker(void)
{
}

CAG_TPTypeChecker::~CAG_TPTypeChecker(void)
{
}

bool CAG_TPTypeChecker::CheckType(CAG_TPSymbolTable::TreeChildrenType& Root, CAG_TPSymbolTable& SymbolTable)
{
	// Return value
	bool Res = true;

	// Derive type for each expression for complete tree, not only statement part
	DeriveType(Root, Res, SymbolTable);

	return Res;
}

void CAG_TPTypeChecker::DeriveType(CAG_TPSymbolTable::TreeChildrenType& Root, bool& Res, CAG_TPSymbolTable& SymbolTable)
{
	for(size_t i=0; i<Root.size(); ++i)
	{
		// Check is it a terminal i.e. there are no children 
		if(Root[i].children.size() == 0)
			DeriveTerminalType(Root[i], Res, SymbolTable);
		else
		{
			// Make sure type of all children is known first
			DeriveType(Root[i].children, Res, SymbolTable);

			// Derive non-terminal type
			DeriveNonTerminalType(Root[i], Res, SymbolTable);
		}
	}
}

void CAG_TPTypeChecker::DeriveTerminalType(CAG_TPSymbolTable::TreeNodeType& Terminal, bool& Res, CAG_TPSymbolTable& SymbolTable)
{
	switch(Terminal.value.id().to_long())
	{
	case CAG_TinyPascalGrammar::RealFactorID:
		Terminal.value.value(CAG_TPSymbolTable::real);
		break;
	case CAG_TinyPascalGrammar::IntegerFactorID:
		Terminal.value.value(CAG_TPSymbolTable::integer);
		break;
	case CAG_TinyPascalGrammar::IdentifierID:
		Terminal.value.value(SymbolTable.GetSymbolType(string(Terminal.value.begin(), Terminal.value.end())));
		break;
	}
}

void CAG_TPTypeChecker::DeriveNonTerminalType(CAG_TPSymbolTable::TreeNodeType& NonTerminal, bool& Res, CAG_TPSymbolTable& SymbolTable)
{
	switch(NonTerminal.value.id().to_long())
	{
	case CAG_TinyPascalGrammar::TermID:
	case CAG_TinyPascalGrammar::ExpressionID:
		DeriveExpressionType(NonTerminal, NonTerminal.children[0], NonTerminal.children[1]);
		break;
	case CAG_TinyPascalGrammar::OpAssignID:
		CheckOpAssignType(NonTerminal.children[0], NonTerminal.children[1], Res);
		break;
	case CAG_TinyPascalGrammar::ForStatementID:
		CheckForLoopType(NonTerminal.children[1], NonTerminal.children[3], Res);
		break;
	}
}

void CAG_TPTypeChecker::DeriveExpressionType(CAG_TPSymbolTable::TreeNodeType& ExpressionNode, 
											 CAG_TPSymbolTable::TreeNodeType& Child1, 
											 CAG_TPSymbolTable::TreeNodeType& Child2)
{
	// Simply: if both sub-expressions are defined and at least one is real, then expression is of type real!
	ExpressionNode.value.value(CAG_TPSymbolTable::undefined);
	if(Child1.value.value() != CAG_TPSymbolTable::undefined && Child2.value.value() != CAG_TPSymbolTable::undefined)
	{
		if(Child1.value.value() == CAG_TPSymbolTable::real || Child2.value.value() == CAG_TPSymbolTable::real)
			ExpressionNode.value.value(CAG_TPSymbolTable::real);
		else ExpressionNode.value.value(CAG_TPSymbolTable::integer);
	}

}

void CAG_TPTypeChecker::CheckOpAssignType(CAG_TPSymbolTable::TreeNodeType& Child1, 
						                  CAG_TPSymbolTable::TreeNodeType& Child2,
										  bool& Res)
{
	if(Child1.value.value() != CAG_TPSymbolTable::undefined && Child2.value.value() != CAG_TPSymbolTable::undefined)
	{
		if(Child1.value.value() == CAG_TPSymbolTable::integer && Child2.value.value() == CAG_TPSymbolTable::real)
			CAG_TPMessageLog::m_WarningLog.push_back("    WARNING: Expression of type real assigned to expression of type integer!");
	}
	else
	{
		CAG_TPMessageLog::m_ErrorLog.push_back("    ERROR: Could not check type for assignment operation!");
		Res = false;
	}
}

void CAG_TPTypeChecker::CheckForLoopType(CAG_TPSymbolTable::TreeNodeType& Child2, 
										 CAG_TPSymbolTable::TreeNodeType& Child4,
										 bool& Res)
{
	// Check that loop variable is of type integer
	if(Child2.children[0].value.value() != CAG_TPSymbolTable::integer)
	{
		CAG_TPMessageLog::m_ErrorLog.push_back("    ERROR: Loop variable must be of type integer!");
		Res = false;
	}

	// Check loop expression to be integer
	if(Child4.value.value() != CAG_TPSymbolTable::integer)
	{
		CAG_TPMessageLog::m_ErrorLog.push_back("    ERROR: Loop expression must be of type integer!");
		Res = false;
	}
}

}}
