#pragma once
#include "AG_TPSymbolTable.h"
#include "AG_TPMessageLog.h"

namespace TinyPascal { namespace Semantics {
class CAG_TPTypeChecker
{
protected:
	//! Derives type for each expression and checks type compatibiliy
	/*! Derives type for each expression and checks type compatibiliy by simply
		traversing the tree and evaluating on type (not value).
	*/
	void DeriveType(CAG_TPSymbolTable::TreeChildrenType& Root, bool& Res, CAG_TPSymbolTable& SymbolTable);

	//! Derives a type of a terminal
	void DeriveTerminalType(CAG_TPSymbolTable::TreeNodeType& Terminal, bool& Res, CAG_TPSymbolTable& SymbolTable);

	//! Derives a type of a non-terminal
	void DeriveNonTerminalType(CAG_TPSymbolTable::TreeNodeType& NonTerminal, bool& Res, CAG_TPSymbolTable& SymbolTable);

	//! Derives type for expression/term
	void DeriveExpressionType(CAG_TPSymbolTable::TreeNodeType& ExpressionNode, 
						      CAG_TPSymbolTable::TreeNodeType& Child1, 
							  CAG_TPSymbolTable::TreeNodeType& Child2);

	//! Checks types for assignment
	void CheckOpAssignType(CAG_TPSymbolTable::TreeNodeType& Child1, 
						   CAG_TPSymbolTable::TreeNodeType& Child2,
						   bool& Res);

	//! Checks for loop types
	void CheckForLoopType(CAG_TPSymbolTable::TreeNodeType& Child2, 
						  CAG_TPSymbolTable::TreeNodeType& Child4,
						  bool& Res);

public:
	//! Default constructor
	CAG_TPTypeChecker(void);

	//! Destructor
	~CAG_TPTypeChecker(void);

	//! Derives type for each expression and checks type compatibility
	bool CheckType(CAG_TPSymbolTable::TreeChildrenType& Root, CAG_TPSymbolTable& SymbolTable);
};
}}
