// ASTView.cpp : implementation file
//

#include "stdafx.h"
#include "TinyPascal.h"
#include "ASTView.h"
#include "Grammar.h"


// CASTView

IMPLEMENT_DYNCREATE(CASTView, CTreeView)

CASTView::CASTView()
{
	m_Font.CreatePointFont(80, _T("Courier New"));
}

CASTView::~CASTView()
{
}

void CASTView::ShowSubTree(CAG_TPSymbolTable::TreeChildrenType &SubTree, HTREEITEM SubTreeRoot)
{
	for(size_t i=0; i<SubTree.size(); ++i)
	{
		// Get rule name
		CString strRuleName(TinyPascal::Parser::CAG_TinyPascalGrammar::GetRuleName((const int)(SubTree[i].value.id().to_long())).c_str());

		// Get type in string format
		CString strType = "";
		if(SubTree[i].value.value()== CAG_TPSymbolTable::real)
			strType = "{type: REAL}";
		else if(SubTree[i].value.value() == CAG_TPSymbolTable::integer)
			strType = "{type: INTEGER}";

		string strValue(SubTree[i].value.begin(), SubTree[i].value.end());
		CString strText = strRuleName + _T(" \"") + CString(strValue.c_str()) + _T("\" ") + strType;

		// Add this rule to the tree control
		TVINSERTSTRUCT tvInsert;
		tvInsert.hParent = SubTreeRoot;
		tvInsert.hInsertAfter = NULL;
		tvInsert.item.mask = TVIF_TEXT;
		tvInsert.item.pszText = strText.GetBuffer();
		HTREEITEM hRoot = GetTreeCtrl().InsertItem(&tvInsert);

		// Add all children of this rule to the tree view
		ShowSubTree(SubTree[i].children, hRoot);

		// Expand it!
		GetTreeCtrl().Expand(hRoot, TVE_EXPAND);
	}
}

void CASTView::ShowAST(CAG_TPSymbolTable::TreeChildrenType &AST)
{
	GetTreeCtrl().DeleteAllItems();

	ShowSubTree(AST, NULL);
}

BEGIN_MESSAGE_MAP(CASTView, CTreeView)
END_MESSAGE_MAP()


// CASTView diagnostics

#ifdef _DEBUG
void CASTView::AssertValid() const
{
	CTreeView::AssertValid();
}

#ifndef _WIN32_WCE
void CASTView::Dump(CDumpContext& dc) const
{
	CTreeView::Dump(dc);
}
#endif
#endif //_DEBUG


// CASTView message handlers

void CASTView::OnInitialUpdate()
{
	CTreeView::OnInitialUpdate();

	// Set font
	GetTreeCtrl().SetFont(&m_Font);

	// Set view to be details
	::SetWindowLong(GetTreeCtrl().m_hWnd, GWL_STYLE, GetWindowLong(GetTreeCtrl().m_hWnd, GWL_STYLE) |
		TVS_HASLINES | TVS_LINESATROOT | TVS_HASBUTTONS);
}
